use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
API

APIs

ASN
AdWords

Alders
Alders'
Andy
AnonymousIP
BR

CN

Canzoneri
Chelluri
Choroba
City
Client
ConnectionType
Continent
Country
Cronin
Database
Dave
Domain
Eilam
Eilam's
Enterprise
Error
Flat
Florian
Fowler
GEOLOCATION

GEONAMES

Generic
GeoIP

GeoIP2
GeoLite

GeoNames

GitHub

Graham
Greg
HTTP
HasIPAddress
HasLocales
HasNames
HasSubdivisions
Hunter
IANA

IP

IP's

IPAddressNotFound
IPv

ISP
ISP

Inc
Insights
Jack
Knop
Knop

Location
MAXMIND
MAXMIND's
Mark
Mateu
MaxMind
MaxMind

MaxMind's
MaxMind's

Michael
Model
NA

Narsimham
OC

Oceania

Olaf
Omni

Oschwald
Oschwald

Oschwald's
Oxfordshire

Patrick
PatrickCronin
PayPal
Postal
Ragwitz
Reader
Record
RepresentedCountry
Role
Rolsky
Rolsky

Rolsky's
SSL

Storey
Subdivision
Tor

Traits
Type
Types
URI

VERSIONING

VPN

WebService
William
YYY

YYYZZZ

ZZZ

anonymizing

bin
choroba
com

contentDeliveryNetwork

de

dialup

docs

downloadable

drolsky
en

ergument

es

foo

fr

geoip

geolocation

geoname_id

github
goschwald
haarg
hostname

ip

ja

lib
libmaxminddb

lookup

maxmind

mfowler
mhunter
mikecanzoneri
nchelluri
oalders
omni

params

pt

rafl
routable

ru

searchEngineSpider

sourced

synced

ua

uk

unpopulated

versioning

web
wstorey
zh

